pub mod migration;

use dioxus::prelude::*;

use crate::util::model::{member::Member, session::Session};

pub fn Members() -> Element {
    let members = use_resource(fetch_members);

    let modal_member: Signal<Option<Member>> = use_signal(|| None);

    rsx! {
        div {
            class: "max-w-3xl w-full",
            h2 { class: "font-bold text-lg", "Leden" },
            match &*members.read_unchecked() {
                Some(Ok(res)) => rsx! {
                    div {
                        class: "overflow-x-auto",
                        table {
                            class: "table table-zebra table-pin-rows",
                            thead {
                                tr {
                                    th { "Relatiecode" }
                                    th { "Naam" }
                                    th { "Registratiecode" }
                                }
                            }
                            tbody {
                                for member in res {
                                    MemberRow { member: member.clone(), modal_member: modal_member }
                                }
                            }
                        }
                    }
                },
                Some(Err(_)) => rsx! { div { "Error while loading members" } },
                None => rsx! { div { "Loading..." } },
            }
            MemberModal { member: modal_member }
        }
    }
}

#[component]
fn MemberRow(member: Member, modal_member: Signal<Option<Member>>) -> Element {
    let member_parse = member.clone();

    let registration_token = member_parse
        .registration_token
        .unwrap_or("None".to_string());

    rsx! {
        tr {
            class: "hover hover:cursor-pointer",
            onclick: move |_| {
                modal_member.set(Some(member.to_owned()))
            },
            th { "{member.id}" }
            td { "{member.name.full}" }
            td { "{registration_token}" }
        }
    }
}

#[server]
async fn fetch_members() -> Result<Vec<Member>, ServerFnError> {
    let user = Session::fetch_current_user().await?;

    if !user.admin {
        return Err(crate::Error::NoPermissions.into());
    }

    let members = Member::fetch_all().await?;
    Ok(members)
}

#[component]
fn MemberModal(member: Signal<Option<Member>>) -> Element {
    let mut groups_focus = use_signal(|| false);

    rsx! {
        dialog {
            class: "modal modal-bottom sm:modal-middle z-50",
            open: member().is_some(),
            div {
                class: "modal-box",
                h3 { class: "text-lg font-bold", "Lid bewerken" },
                if let Some(m) = member() {
                    div {
                        div {
                            class: "grid gap-3 sm:grid-flow-col sm:grid-rows-3 sm:grid-cols-2",
                            label {
                                class: "form-control w-full",
                                div {
                                    class: "label",
                                    span { class: "label-text", "Naam" }
                                }
                                b { class: "select-all", "{m.name.full}" },
                            }
                            label {
                                class: "form-control w-full",
                                div {
                                    class: "label",
                                    span { class: "label-text", "Relatiecode" }
                                }
                                b { class: "select-all", "{m.id}" },
                            }
                            label {
                                class: "form-control w-full",
                                div {
                                    class: "label",
                                    span { class: "label-text", "Registratiecode" }
                                }
                                b { class: "select-all", {m.registration_token.unwrap_or("geen".to_string())} },
                            }
                            label {
                                class: "form-control w-full",
                                div {
                                    class: "label",
                                    span { class: "label-text", "Uur" }
                                }
                                b { class: "select-all", {m.hours.join(", ")} },
                            }
                            label {
                                class: "form-control w-full",
                                div {
                                    class: "label",
                                    span { class: "label-text", "Diploma" }
                                }
                                b { class: "select-all", {m.diploma.unwrap_or("geen".to_string())} },
                            }
                        }
                        div {
                            class: "mt-5",
                            div {
                                class: "dropdown w-full",
                                label {
                                    class: "form-control w-full",
                                    div {
                                        class: "label",
                                        span { class: "label-text", "Groepen" }
                                    }
                                    input {
                                        r#type: "text",
                                        class: "input input-bordered",
                                        onfocusin: move |_| {
                                            groups_focus.set(true);
                                        },
                                        onfocusout: move |_| {
                                            groups_focus.set(false);
                                        },
                                    }
                                }
                                ul {
                                    class: "dropdown-content menu bg-base-100 rounded-box z-[1] p-2 shadow w-full",
                                    li { "Kader" }
                                    li { "Wedstrijden" }
                                    li { "Bestuur" }
                                }
                            }
                        }
                    }
                } else {
                    "Geen lid geselecteerd"
                }
                div {
                    class: "modal-action",
                    button {
                        class: "btn",
                        onclick: move |_| member.set(None),
                        "Annuleren"
                    }
                }
            }
            label {
                class: "modal-backdrop bg-black/50",
                onclick: move |_| member.set(None)
            }
        }
    }
}
